@@ -1,7 +1,7 @@
 /* Sample prototype for a trampoline. */
 
 /*
- * Copyright 1995-1998 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1998 Bruno Haible, <bruno@clisp.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -42,6 +42,9 @@ register void* env __asm__("r11");
 #ifdef __convex__
 register void* env __asm__("s0");
 #endif
+#ifdef __s390__
+register void* env __asm__("r0");
+#endif
 
 int tramp ()
 { env = data;
