@@ -1,5 +1,5 @@
 /*
- * Copyright 1995-1999 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2003, 2005 Bruno Haible, <bruno@clisp.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -17,13 +17,7 @@
 __va_struct_buffer_t __va_struct_buffer;
 
 int /* no return type, since this never returns */
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __va_error1 (enum __VAtype start_type, enum __VAtype return_type)
-#else
-__va_error1 (start_type, return_type)
-  enum __VAtype start_type;
-  enum __VAtype return_type;
-#endif
 {
   /* If you see this, fix your code. */
   fprintf (stderr, "vacall: va_start type %d and va_return type %d disagree.\n",
@@ -35,12 +29,7 @@ __va_error1 (start_type, return_type)
 }
 
 int /* no return type, since this never returns */
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __va_error2 (unsigned int size)
-#else
-__va_error2 (size)
-  unsigned int size;
-#endif
 {
   /* If you see this, increase __VA_ALIST_WORDS: */
   fprintf (stderr, "vacall: struct of size %u too large for pcc struct return.\n",
