@@ -14,9 +14,9 @@
 ;
 ; You should have received a copy of the GNU General Public License
 ; along with this program; if not, write to the Free Software Foundation,
-; Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+; Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ;
-; $Id: oracle.lisp,v 1.16 2004/01/27 15:30:41 sds Exp $
+; $Id: oracle.lisp,v 1.21 2005/05/20 20:03:51 haible Exp $
 
 (defpackage "ORACLE"
   (:documentation
@@ -98,7 +98,7 @@ to-sqlval to-string update-row valid-symbol
 
 ; CONNECT
 
-(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) (truncate-ok t))
+(defun connect (user password server &optional schema (auto-commit t) (prefetch-buffer-bytes 0) (long-len -1) truncate-ok)
 "(ORACLE::CONNECT user password server &optional schema auto-commit prefetch-buffer-bytes long-len truncate-ok)
 
 Connect to an Oracle database.  All subsequent operations will affect
@@ -159,8 +159,8 @@ Returns: T if a cached connection was re-used (NIL if a new connection
 	    (when (not (lisp-truth (oracle_success conn-handle)))
 		  (oracle_disconnect conn-handle) ; Don't check for error here
 		  (remhash hkey *oracle-connection-cache*)
-		  (setf conn nil)
-		  (setf *oracle-connection* nil))))
+		  (setf conn nil
+				*oracle-connection* nil))))
 
     (when (null conn)
           ; Connect to database
@@ -171,9 +171,9 @@ Returns: T if a cached connection was re-used (NIL if a new connection
                    ; Failed all attempts; give up
                   (db-error (oracle_last_error handle)))
             ; OK: cache the connection
-            (setf conn (make-db :connection handle :hkey hkey))
-            (setf (gethash hkey *oracle-connection-cache*) conn)
-            (setf result nil)))
+            (setf conn (make-db :connection handle :hkey hkey)
+				  (gethash hkey *oracle-connection-cache*) conn
+				  result nil)))
     ; Set current connection
     (setf *oracle-connection* conn)
     result))
@@ -241,9 +241,9 @@ for SELECT statements.
                    sql
                    (hash-to-sqlparam-array params)
                    (c-truth (not is-select)))
-  (setf (db-fetch-called *oracle-connection*) nil)
-  (setf (db-pending-row *oracle-connection*) nil)
-  (setf (db-colinfo *oracle-connection*) nil)
+  (setf (db-fetch-called *oracle-connection*) nil
+		(db-pending-row *oracle-connection*) nil
+		(db-colinfo *oracle-connection*) nil)
   (check-success)
 
   ; Get the row count for the result
@@ -357,8 +357,8 @@ Arguments: none
     (cond
      ; Use pending look-ahead row and reset, else do a "real" fetch
      ((db-pending-row *oracle-connection*)
-      (setf result (row-to-result (db-pending-row *oracle-connection*) result-type))
-      (setf (db-pending-row *oracle-connection*) nil))
+      (setf result (row-to-result (db-pending-row *oracle-connection*) result-type)
+			(db-pending-row *oracle-connection*) nil))
 
      ; Check if already at EOF from previous fetches
      ((and (db-fetch-called *oracle-connection*)
@@ -474,6 +474,12 @@ Arguments: none
         ; Convert C truth to Lisp for export
         (map-into result
                   #'(lambda (col)
+					  ; Oracle identifies FLOAT using special value -127 for scale,
+					  ; (which is irrelevant for floats).  In this case, map to "FLOAT"
+					  ; for type name and NIL for scale.  Precision will be given in bits
+					  ; as ANSI specifies
+					  (when (and (equal (sqlcol-type col) "NUMBER") (= (sqlcol-scale col) -127) (not (= 0 (sqlcol-precision col))))
+						    (setf (sqlcol-type col) "FLOAT"))
                       (setf (sqlcol-null_ok col) (lisp-truth (sqlcol-null_ok col)))
                       col)
                   result)
@@ -763,7 +769,7 @@ Argument: none
     (cond ((null val) nil)
           ((find dtype '("NUMBER" "INTEGER" "FLOAT") :test #'equal)
            (read-from-string val))
-          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "BLOB" "CLOB" "BFILE") :test #'equal)
+          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "BLOB" "CLOB" "BFILE" "ROWID DESC") :test #'equal)
            val)
           (t (db-error (cat "Unsupported data type '" dtype "'"))))))
 
