@@ -17,9 +17,9 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-   $Id: orafns.c,v 1.7 2003/08/02 16:35:09 hin Exp $
+   $Id: orafns.c,v 1.15 2005/05/20 20:03:51 haible Exp $
 
 */
 
@@ -40,6 +40,8 @@
 #define DEFAULT_PREFETCH_BYTES  65536
 /* Size of per-connection error buffer */
 #define ERRBUF_BYTES            100000
+/* Truncate echoed parameter values to this length */
+#define ERRBUF_PARAM_TRUNC      500
 /* Default size of LONG fetch */
 #define DEFAULT_LONG_LEN		500000
 
@@ -61,7 +63,8 @@ static int              rollback(struct db_conn *);
 static int              set_auto_commit(struct db_conn *, int);
 
 /* Purely local routines */
-static void      append_oci_error(char *, OCIError *);
+static void      append_oci_error(char *, void *, int);
+static void      append_indicator(struct db_conn *, char *);
 static int       init_session(struct db_conn *, char *, char *, char *, char*, int);
 static int       get_cols(struct db_conn *);
 static int       get_param_attr(CONST dvoid *, int, struct db_conn *, dvoid *, ub4 *, char *, ub4);
@@ -283,7 +286,7 @@ static int fetch_row(struct db_conn * db)
   if ( fetch_status != OCI_SUCCESS && fetch_status != OCI_SUCCESS_WITH_INFO ) {
     /* Error fetching */
     sprintf(db->errmsg, "Fetch error %d:\n", fetch_status);
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
   
@@ -301,7 +304,7 @@ static int fetch_row(struct db_conn * db)
       sprintf(db->errmsg, "Error getting error status checking for EOF\n");
       return db->success = 0;
     }
-    
+
     /* Check for EOF */
     if ( errcode == 1403 ) {
       db->eof = 1;
@@ -324,17 +327,39 @@ static int fetch_row(struct db_conn * db)
 	  ; /* Ignore */
     else {
       sprintf(db->errmsg, "Unknown Oracle warning %d\n", errcode);
-      append_oci_error(db->errmsg, db->err);
+      append_oci_error(db->errmsg, db->err, 0);
       return db->success = 0;
     }
   }
 
+  /* If row-level truncation indicator given, make sure it was not because of
+	 a ROWID being in the result (which will always trigger that indicator).
+	 In this case, scan the result columns for a ROWID type, and if one appears,
+	 turn off the global truncation and NULL flags */
+  if ( truncated )
+	for ( n=0; n < db->ncol; n++ ) {
+	  col = &db->columns[n];
+	  if ( col->dtype == SQLT_RDD ) {
+		truncated = 0;
+		null = 0;
+		/* Platform-dependent ROWID Descriptor gives its length in the indicator, usually 18 bytes */
+		col->indicator = 0; /* Reset it as if not truncated */
+	  }
+	}
+	  
+  /* Positive indicator means truncated because it was that large, -2 means truncated from value so large, size does not fit in a SB2 */
+  for ( n=0; n < db->ncol; n++ ) {
+    col = &db->columns[n];
+	if ( col->indicator > 0 || col->indicator == -2 )
+	  truncated = 1;
+  }
+
   /* If data was truncated and that is not allowed, raise an error */
   if ( truncated && ! db->truncate_ok ) {
 	char * defind = db->long_len == DEFAULT_LONG_LEN ? "default" : "specified";
 	sprintf(db->errmsg,
 			"Size of a LONG column exceeds %s maximum of %d.\nYou must either increase size of LONG/BLOB fetch buffer, or set flag allowing truncation.\n", defind, db->long_len);
-	append_oci_error(db->errmsg, db->err);
+	append_oci_error(db->errmsg, db->err, 0);
 	return db->success = 0;
   }
   
@@ -369,12 +394,12 @@ static int fetch_row(struct db_conn * db)
 					"Size of BLOB/CLOB column '%s' exceeds %s maximum of %d.\n"
 					"You must either increase size of LONG/BLOB fetch buffer, or set flag allowing truncation.\n",
 					col->name, defind, db->long_len);
-			append_oci_error(db->errmsg, db->err);
+			append_oci_error(db->errmsg, db->err, 0);
 			return db->success = 0;
 		  }
 		  else if ( lob_read_status != OCI_SUCCESS ) {
 			sprintf(db->errmsg, "Error reading LOB column '%s'\n", col->name);
-			append_oci_error(db->errmsg, db->err);
+			append_oci_error(db->errmsg, db->err, 0);
 			return db->success = 0;
 		  }
 		
@@ -384,9 +409,10 @@ static int fetch_row(struct db_conn * db)
 	}
     else if ( col->indicator == -1 )
       r->is_null = 1;
-    else {
+    else if ( col->indicator < -2 ) {
       sprintf(db->errmsg, "Bad indicator value %d for column %d\n", col->indicator, n);
-      append_oci_error(db->errmsg, db->err);
+      append_oci_error(db->errmsg, db->err, 0);
+	  return db->success = 0;
     }
   }
   
@@ -432,7 +458,7 @@ static int get_cols(struct db_conn * db)
   if ( attr_status != OCI_SUCCESS ) {
     /* Error getting param info */
     sprintf(db->errmsg, "Error getting param count:\n");
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
 
@@ -445,6 +471,10 @@ static int get_cols(struct db_conn * db)
 
   do {
 
+	/* Oracle requires a sb2, not an ub1, to receive the OCI_ATTR_PRECISION for implicit describes,
+	   even though in the data dictionary it is a one-byte unsigned value.  Got that?  I didn't.  */
+	sb2 tmp_precision = 0;
+
     ncol++;
     param_status = OCIParamGet(db->stmt, OCI_HTYPE_STMT, db->err, (dvoid **) &param, ncol);
 
@@ -479,7 +509,7 @@ static int get_cols(struct db_conn * db)
       else {
         /* A real error */
         sprintf(db->errmsg, "Error getting param col %d:\n", ncol);
-        append_oci_error(db->errmsg, db->err);
+        append_oci_error(db->errmsg, db->err, 0);
         return db->success = 0;
       }
     }
@@ -496,11 +526,12 @@ static int get_cols(struct db_conn * db)
     col = &db->columns[ncol-1];
 
     /* Get param attributes */
+	/* Note for OCI_ATTR_PRECISION we receive the value in an (sb2 *) rather than (ub1 *) for our "implicit describe" */
     success = 1;
     success = success && get_param_attr(param, ncol, db, &colnamep,       &colname_len, "name",      OCI_ATTR_NAME);
     success = success && get_param_attr(param, ncol, db, &col->dtype,     0,            "data type", OCI_ATTR_DATA_TYPE);
     success = success && get_param_attr(param, ncol, db, &col->dsize,     0,            "size",      OCI_ATTR_DATA_SIZE);
-    success = success && get_param_attr(param, ncol, db, &col->precision, 0,            "precision", OCI_ATTR_PRECISION);
+    success = success && get_param_attr(param, ncol, db, &tmp_precision,  0,            "precision", OCI_ATTR_PRECISION);
     success = success && get_param_attr(param, ncol, db, &col->scale,     0,            "scale",     OCI_ATTR_SCALE);
     success = success && get_param_attr(param, ncol, db, &col->null_ok,   0,            "is-null",   OCI_ATTR_IS_NULL);
     if ( ! success )
@@ -511,6 +542,13 @@ static int get_cols(struct db_conn * db)
     colname[colname_len] = '\0';
     col->name = (char *) strdup(colname);
 
+	/* Copy precision.  For grins, check that it will fit in an ub1 */
+	if ( tmp_precision < 0 || tmp_precision > 127 ) {
+	  sprintf(db->errmsg, "Precision [%d] out of bounds for column '%s'", col->name);
+	  return db->success = 0;
+	}
+	col->precision = tmp_precision;
+	
     /*
       printf("Col %2d: type is %-10s name = %-20s size = %4d precision = %2d scale = %2d null-ok=%d\n",
              ncol, decode_data_type(col->dtype), col->name, col->dsize, col->precision, col->scale, col->null_ok);
@@ -551,7 +589,7 @@ static int get_cols(struct db_conn * db)
                                    &col->indicator, &col->nfetched, &col->rcode, OCI_DEFAULT);
     if ( define_status != OCI_SUCCESS ) {
       sprintf(db->errmsg, "Error setting up define for column '%s':\n", col->name);
-      append_oci_error(db->errmsg, db->err);
+      append_oci_error(db->errmsg, db->err, 0);
       return db->success = 0;
     }
   }
@@ -601,7 +639,7 @@ static int commit(struct db_conn * db)
   status = OCITransCommit(db->svc, db->err, OCI_DEFAULT);
   if ( status != OCI_SUCCESS ) {
     sprintf(db->errmsg, "Error commiting transaction:\n");
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
 
@@ -628,7 +666,7 @@ static int rollback(struct db_conn * db)
   status = OCITransRollback(db->svc, db->err, OCI_DEFAULT);
   if ( status != OCI_SUCCESS ) {
     sprintf(db->errmsg, "Error rolling back transaction:\n");
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
 
@@ -690,15 +728,16 @@ static int exec_sql(struct db_conn * db, char * sql, struct sqlparam ** params,
   }
   if ( OCI_SUCCESS !=  OCIHandleAlloc(db->env, (dvoid **) &db->stmt, OCI_HTYPE_STMT, 0, 0) ) {
     sprintf(db->errmsg, "Error allocating statement handle\n");
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
 
   /* "Prepare" the statement - this rarely returns an error */
   status = OCIStmtPrepare(db->stmt, db->err, sql, strlen(sql), OCI_NTV_SYNTAX, OCI_DEFAULT);
   if ( status != OCI_SUCCESS ) {
-    sprintf(db->errmsg, "Error parsing SQL text:\n%s\n", sql);
-    append_oci_error(db->errmsg, db->err);
+    sprintf(db->errmsg, "Error parsing SQL %s:\n---\n%s\n---\n", is_command ? "command" : "query", sql);
+    append_oci_error(db->errmsg, db->err, 0);
+	append_indicator(db, sql);
     return db->success = 0;
   }
 
@@ -742,7 +781,7 @@ static int exec_sql(struct db_conn * db, char * sql, struct sqlparam ** params,
     if ( status != OCI_SUCCESS ) {
       sprintf(db->errmsg, "Error binding SQL input parameter '%s' to value '%s' in query:\n---\n%s\n---\n",
               pname, data, sql);
-      append_oci_error(db->errmsg, db->err);
+      append_oci_error(db->errmsg, db->err, 0);
       return db->success = 0;
     }
 
@@ -758,44 +797,36 @@ static int exec_sql(struct db_conn * db, char * sql, struct sqlparam ** params,
 
   if ( OCI_SUCCESS != status ) {
 
-    ub4 err_offset = 0;
-
     sprintf(db->errmsg, "Error executing SQL %s:\n---\n%s\n---\n", is_command ? "command" : "query", sql);
 
-    /* Also show what params were given */
+    /* Also show what params were given.  Since parameter values may be really big, and our error buffer
+	   is fixed, truncate the parameter values to about 500 bytes (while noting the length if they were
+	   truncated).  This will allow them to be displayed in a reasonably readable way, while at the
+	   same time, showing the length will enable the user to diagnose "value too large for column" problems. */
     if ( nparam ) {
       int i;
-      char buf[50000], buf2[10000];
+      char buf[ERRBUF_BYTES], buf2[10000];
 
       sprintf(buf, "SQL bind parameters (%d) given:\n", nparam);
       strcat(db->errmsg, buf);
       params = params_save;
       for (i=0; i < nparam; i++) {
-        sprintf(buf, "'%s' -> '%s'\n", params[i]->name, params[i]->value.data);
+		char * pdata = params[i]->value.data;
+		int plen = strlen(pdata);
+		if ( plen > ERRBUF_PARAM_TRUNC ) {
+		  sprintf(buf2, "[String of length %d (truncated)] ", plen);
+		  strncat(buf2, pdata, ERRBUF_PARAM_TRUNC);
+		  strcat(buf2, "...");
+		  pdata = buf2;
+		}
+        sprintf(buf, "'%s' -> '%s'\n", params[i]->name, pdata);
         strcat(db->errmsg, buf);
       }
       strcat(db->errmsg, "---\n");
     }
     
-    append_oci_error(db->errmsg, db->err);
-
-    /* Get the parse error offset so we can print an indicator as to
-       where the error occurred */
-    status = OCIAttrGet(db->stmt, OCI_HTYPE_STMT, (dvoid *) &err_offset, (ub4 *) 0, OCI_ATTR_PARSE_ERROR_OFFSET, db->err);
-    if ( OCI_SUCCESS != status ) {
-      strcat(db->errmsg, "Could not get parse error offset:\n");
-      append_oci_error(db->errmsg, db->err);
-    }
-    else {
-      char p[100];
-      sprintf(p, "At character %d, near ==> indicator in:\n----\n", err_offset);
-      strcat(db->errmsg, p);
-
-      strncat(db->errmsg, sql, err_offset);
-      strcat(db->errmsg, "\n==> ");
-      strcat(db->errmsg, sql + err_offset);
-      strcat(db->errmsg, "\n----\n");
-    }
+    append_oci_error(db->errmsg, db->err, 0);
+	append_indicator(db, sql);
     return db->success = 0;
   }
 
@@ -821,7 +852,7 @@ static int exec_sql(struct db_conn * db, char * sql, struct sqlparam ** params,
     attr_status = OCIAttrGet((dvoid *) db->stmt, OCI_HTYPE_STMT, &db->rows_affected, 0, OCI_ATTR_ROW_COUNT, db->err);
     if ( attr_status != OCI_SUCCESS ) {
       sprintf(db->errmsg, "Error getting row count for command:\n");
-      append_oci_error(db->errmsg, db->err);
+      append_oci_error(db->errmsg, db->err, 0);
       return db->success = 0;
     }
   }
@@ -851,7 +882,7 @@ static int init_session(struct db_conn * db, char *user, char * schema, char *pa
   /* If already connected, drop and re-connect */
   if ( db->env )
     disconnect(db);
-
+  
   /* Clear out results */
   *db->errmsg = '\0';
   db->env = 0;
@@ -872,27 +903,39 @@ static int init_session(struct db_conn * db, char *user, char * schema, char *pa
   }
   
   /* Init Oracle library */
-  /* TODO: use mode OCI_OBJECT so that we can describe tables w/ BLOB cols */
+  /* If the Oracle 8 version is really old (< 8.1.5) use these old calls instead of OCIEnvCreate */
+#if 0
   status = OCIInitialize(OCI_OBJECT,
                          0,         /* Context for mode OCI_OBJECT */
                          0,         /* Our malloc() func */
                          0,         /* Our realloc() func */
                          0);        /* Our free() func */
-
-  /* Init an Oracle "environment handle" */
   status = OCIEnvInit(&db->env,     /* Returned: new handle */
                       OCI_DEFAULT,  /* Use mutexes */
                       0,            /* Our memory size */
                       0);           /* Returned: alloc'ed memory */
+#endif
+
+  /* Create the environment */
+  status = OCIEnvCreate(&db->env,     /* Returned: new handle */
+						OCI_OBJECT,   /* Use object features */
+                        0,            /* Context */
+                        0,            /* Our malloc() func */
+                        0,            /* Our realloc() func */
+                        0,            /* Our free() func */
+						0,            /* Our memory size */
+						0);           /* Returned: alloc'ed memory */
   if ( OCI_SUCCESS != status ) {
-    sprintf(db->errmsg, "Error initializing OCI library: OCIEnvInit() returned %s", status);
+    sprintf(db->errmsg, "Error initializing OCI library: OCIEnvCreate/OCIEnvInit() returned %d:\n", status);
+	append_oci_error(db->errmsg, db->env, 1);
     return db->success = 0;
   }
 
   /* Get an error handle */
   status = OCIHandleAlloc(db->env, (dvoid **) &db->err, OCI_HTYPE_ERROR, 0, 0);
   if ( OCI_SUCCESS != status ) {
-    sprintf(db->errmsg, "Error initializing OCI error handle: OCIHandleAlloc(OCI_HTYPE_ERROR) returned %s", status);
+    sprintf(db->errmsg, "Error initializing OCI error handle: OCIHandleAlloc(OCI_HTYPE_ERROR) returned %d:\n", status);
+	append_oci_error(db->errmsg, db->env, 1);
     OCIHandleFree(db->env, OCI_HTYPE_ENV);
     db->env = 0;
     return db->success = 0;
@@ -903,7 +946,7 @@ static int init_session(struct db_conn * db, char *user, char * schema, char *pa
                     user, strlen(user), password, strlen(password), sid, strlen(sid));
   if ( OCI_SUCCESS != status ) {
     sprintf(db->errmsg, "Error logging on to Oracle service '%s' as user '%s':\n", sid, user);
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     OCIHandleFree(db->env, OCI_HTYPE_ENV);
     db->env = 0;
     return db->success = 0;
@@ -940,7 +983,7 @@ static int init_session(struct db_conn * db, char *user, char * schema, char *pa
   status = OCIAttrSet(db->stmt, OCI_HTYPE_STMT, &attval, 0, OCI_ATTR_PREFETCH_MEMORY, db->err);
   if ( status != OCI_SUCCESS ) {
     sprintf(db->errmsg, "Error %d setting prefetch memory:\n", status);
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
     
@@ -955,7 +998,7 @@ static int init_session(struct db_conn * db, char *user, char * schema, char *pa
   status = OCIAttrSet(db->stmt, OCI_HTYPE_STMT, &attval, 0, OCI_ATTR_PREFETCH_ROWS, db->err);
   if ( status != OCI_SUCCESS ) {
     sprintf(db->errmsg, "Error settgin prefetch row count:\n");
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
 
@@ -974,7 +1017,7 @@ static int get_param_attr(CONST dvoid * param, int ncol, struct db_conn * db, dv
   if ( attr_status != OCI_SUCCESS ) {
     /* Error getting param info */
     sprintf(db->errmsg, "Error getting column %s for col %d:\n", prompt, ncol);
-    append_oci_error(db->errmsg, db->err);
+    append_oci_error(db->errmsg, db->err, 0);
     return db->success = 0;
   }
 
@@ -984,8 +1027,9 @@ static int get_param_attr(CONST dvoid * param, int ncol, struct db_conn * db, dv
 
 /* ------------------------------------------------------------------------------------------------------------- */
 
-/* Append Oracle error to our specific error */
-static void append_oci_error(char *errbuf, OCIError * err)
+/* Append Oracle error to our specific error.  Error pointer is
+   usually (OCIError *) but sometimes (OCIEnv *), if isenv true */
+static void append_oci_error(char *errbuf, void * err, int isenv)
 {
   sword status      = 0;
   char *dummy = 0;
@@ -997,10 +1041,18 @@ static void append_oci_error(char *errbuf, OCIError * err)
     
   /* Hack: this is highly language dependent */
   char * leading = "Error while trying to retrieve text for error";
+
+  /* Do nothing if don't have a handle */
+  if ( ! err )
+	return;
     
-  status = OCIErrorGet(err, 1, dummy, &errcode, buf, sizeof buf, OCI_HTYPE_ERROR);
-  if ( status )
+  status = OCIErrorGet(err, 1, dummy, &errcode, buf, sizeof buf, isenv ? OCI_HTYPE_ENV : OCI_HTYPE_ERROR);
+  if ( status ) {
+	char reason[1000];
+	sprintf(reason, "[Could not get error message from Oracle: OCIErrorGet() returned %d]\n", status);
+    strcat(errbuf, reason);
     return;
+  }
 
   /* Append Oracle error */
   strcat(errbuf, buf);
@@ -1022,6 +1074,33 @@ static void append_oci_error(char *errbuf, OCIError * err)
 
 }
 
+/* ------------------------------------------------------------------------------------------------------------- */
+/* Append error location indicator to error message */
+void append_indicator(struct db_conn * db, char * sql)
+{
+    ub2		err_offset = 0;
+	sword	status  = 0;
+					  
+    /* Get the parse error offset so we can print an indicator as to
+       where the error occurred */
+    status = OCIAttrGet(db->stmt, OCI_HTYPE_STMT, (dvoid *) &err_offset, (ub4 *) 0, OCI_ATTR_PARSE_ERROR_OFFSET, db->err);
+    if ( OCI_SUCCESS != status ) {
+      strcat(db->errmsg, "Could not get parse error offset:\n");
+      append_oci_error(db->errmsg, db->err, 0);
+    }
+    else {
+      char p[100];
+      sprintf(p, "At character %d, near ==> indicator in:\n----\n", err_offset);
+      strcat(db->errmsg, p);
+
+      strncat(db->errmsg, sql, err_offset);
+      strcat(db->errmsg, "\n==> ");
+      strcat(db->errmsg, sql + err_offset);
+      strcat(db->errmsg, "\n----\n");
+    }
+}
+
+
 /* ------------------------------------------------------------------------------------------------------------- */
 
 /* Decode data type code */
@@ -1029,22 +1108,24 @@ static char * decode_data_type(int dtype)
 {
   static char buf[100];
   switch (dtype) {
-  case 1:   return "VARCHAR2";
-  case 2:   return "NUMBER";
-  case 3:   return "INTEGER";
-  case 4:   return "FLOAT";
-  case 8:   return "LONG";
-  case 11:  return "ROWID";
-  case 12:  return "DATE";
-  case 23:  return "RAW";
-  case 24:  return "LONG RAW";
-  case 96:  return "CHAR";
-  case 104: return "ROWID DESC";
-  case 105: return "MLSLABEL";
-  case 108: return "User defined";
-  case 111: return "REF";
-  case 112: return "CLOB";
-  case 113: return "BLOB";
+  case 1:   return "VARCHAR2";             /* SQLT_CHR */
+  case 2:   return "NUMBER";               /* SQLT_NUM */
+  case 3:   return "INTEGER";              /* SQLT_INT */
+  case 4:   return "FLOAT";                /* SQLT_FLT */
+  case 8:   return "LONG";                 /* SQLT_LNG */
+  case 11:  return "ROWID";                /* SQLT_RID -- through Oracle 7 */
+  case 12:  return "DATE";                 /* SQLT_DAT */
+  case 23:  return "RAW";                  /* SQLT_BIN */
+  case 24:  return "LONG RAW";             /* SQLT_LBI */
+  case 96:  return "CHAR";                 /* SQLT_AFC (ANSI Fixed Char) */
+  case 104: return "ROWID DESC";           /* SQLT_RDD (RowId Descriptor) */
+  case 105: return "MLSLABEL";             /* SQLT_LAB */
+  case 108: return "User defined";         /* SQLT_NTY (Named Type) */
+  case 111: return "REF";                  /* ? */
+  case 112: return "CLOB";                 /* SQLT_CLOB - Character LOB */
+  case 113: return "BLOB";                 /* SQLT_BLOB - Binary LOB */
+  case 114: return "BFILE";                /* SQLT_BFILEE - Binary file LOB */
+  case 115: return "CFILE";                /* SQLT_CFILEE - Character file LOB */
   }
   sprintf("<Unknown type %d>", buf);
   return buf;
@@ -1077,6 +1158,9 @@ static int fetch_data_len(int dtype, int dlen, int long_len)
   case 4:   /* FLOAT (SQLT_FLT) */
     return 50;
     
+  case 104: /* ROWID DESC (SQLT_RDD) */
+	return 50;
+
   case 8:   /* LONG (SQLT_LNG) */
   case 112: /* CLOB (SQLT_CLOB) */
   case 113: /* BLOB (SQLT_BLOB) */
