@@ -13,7 +13,8 @@
   (do-all-symbols (sym) (remprop sym 'sys::definition))
   (when (fboundp 'clos::install-dispatch)
     (do-all-symbols (sym)
-      (when (and (fboundp sym) (clos::generic-function-p (symbol-function sym)))
+      (when (and (fboundp sym)
+                 (typep (symbol-function sym) clos::<standard-generic-function>))
         (let ((gf (symbol-function sym)))
           (when (clos::gf-never-called-p gf)
             (clos::install-dispatch gf)
@@ -26,13 +27,16 @@
 ;; Saves the current memory contents.
 ;; This function works only when compiled!
 (defun saveinitmem (&optional (filename "lispinit.mem")
-                    &key ((:quiet *quiet*) nil) init-function)
-  (let* ((old-driver *driver*)
+                    &key ((:quiet *quiet*) nil) init-function verbose
+                    keep-global-handlers
+                    ((:start-package *package*) *package*)
+                    (locked-packages *system-package-list*))
+  (let* ((old-driver *driver*) old-global-handlers
+         (fn (merge-pathnames filename #.(make-pathname :type "mem")))
          (*driver*
            #'(lambda ()
-               (declare (special *command-index* *home-package*
-                                 *active-restarts* *condition-restarts*
-               )        )
+               ;(declare (special *command-index* *home-package*
+               ;                  *active-restarts* *condition-restarts*))
                ;; Reset a few special variables. This must happen in the
                ;; fresh session, not in the old session, because that would
                ;; temporarily disable error handling in the old session.
@@ -52,14 +56,19 @@
                      *active-restarts* nil
                      *condition-restarts* nil
                      *command-index* 0
-                     *home-package* nil
-               )
+                     *home-package* nil)
                (setq *driver* old-driver)
                (when init-function (funcall init-function))
-               (funcall *driver*)
-             )
-        ))
-    (savemem filename)
-  )
-  (room nil)
-)
+               (funcall *driver*))))
+    (unless keep-global-handlers
+      (setq old-global-handlers ; disable and save all global handlers
+            (ext::set-global-handler nil nil)))
+    (setf (package-lock locked-packages) t)
+    (savemem fn)
+    (when old-global-handlers   ; re-install all global handlers
+      (ext::set-global-handler old-global-handlers nil))
+    (when verbose
+      (fresh-line)
+      (format t (TEXT "Wrote the memory image into ~A") fn)
+      (terpri)))
+  (room nil))
